# pylint: disable=line-too-long
"""The select commands module.

These commands are used in the following models:
MDO3

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - SELect:B<x> {OFF|ON|0|1}
    - SELect:B<x>?
    - SELect:BUS<x> {OFF|ON|0|1}
    - SELect:BUS<x>?
    - SELect:CH<x> {ON|OFF|1|0}
    - SELect:CH<x>?
    - SELect:CONTROl {CH<x> |MATH|REF<x>|BUS<x>|D<x>|RF_AMPlitude|RF_FREQuency |RF_PHASe|RF_NORMal|RF_AVErage| RF_MAXHold|RF_MINHold}
    - SELect:CONTROl?
    - SELect:D<x> {<NR1>|OFF|ON}
    - SELect:D<x>?
    - SELect:DAll {ON|OFF|0|1}
    - SELect:MATH {ON|OFF|<NR1>}
    - SELect:MATH1 {ON|OFF|<NR1>}
    - SELect:MATH1?
    - SELect:MATH?
    - SELect:REF<x> {ON|OFF|<NR1>}
    - SELect:REF<x>?
    - SELect:RF_AVErage {OFF|ON|0|1}
    - SELect:RF_AVErage?
    - SELect:RF_MAXHold {OFF|ON|0|1}
    - SELect:RF_MAXHold?
    - SELect:RF_MINHold {OFF|ON|0|1}
    - SELect:RF_MINHold?
    - SELect:RF_NORMal {OFF|ON|0|1}
    - SELect:RF_NORMal?
    - SELect?
"""  # noqa: E501
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedChannel,
    ValidatedDigitalBit,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class SelectRfNormal(SCPICmdWrite, SCPICmdRead):
    """The ``SELect:RF_NORMal`` command.

    **Description:**
        - This command switches the frequency domain Normal trace display on or off in the frequency
          domain graticule.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:RF_NORMal?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:RF_NORMal?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:RF_NORMal value`` command.

    **SCPI Syntax:**

    ::

        - SELect:RF_NORMal {OFF|ON|0|1}
        - SELect:RF_NORMal?

    **Info:**
        - ``OFF`` or 0 turns the frequency domain Normal trace display off.
        - ``ON`` or 1 turns it on.
    """


class SelectRfMinhold(SCPICmdWrite, SCPICmdRead):
    """The ``SELect:RF_MINHold`` command.

    **Description:**
        - This command switches the frequency domain Min Hold trace display on or off in the
          frequency domain graticule.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:RF_MINHold?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:RF_MINHold?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:RF_MINHold value`` command.

    **SCPI Syntax:**

    ::

        - SELect:RF_MINHold {OFF|ON|0|1}
        - SELect:RF_MINHold?

    **Info:**
        - ``OFF`` or 0 turns the frequency domain Min Hold display off.
        - ``ON`` or 1 turns it on.
    """


class SelectRfMaxhold(SCPICmdWrite, SCPICmdRead):
    """The ``SELect:RF_MAXHold`` command.

    **Description:**
        - This command switches the frequency domain Max Hold trace display on or off in the
          frequency domain graticule.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:RF_MAXHold?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:RF_MAXHold?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:RF_MAXHold value`` command.

    **SCPI Syntax:**

    ::

        - SELect:RF_MAXHold {OFF|ON|0|1}
        - SELect:RF_MAXHold?

    **Info:**
        - ``OFF`` or 0 turns the frequency domain Max Hold trace display off.
        - ``ON`` or 1 turns it on.
    """


class SelectRfAverage(SCPICmdWrite, SCPICmdRead):
    """The ``SELect:RF_AVErage`` command.

    **Description:**
        - This command switches the RF Average trace display on or off in the frequency domain
          graticule.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:RF_AVErage?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:RF_AVErage?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:RF_AVErage value`` command.

    **SCPI Syntax:**

    ::

        - SELect:RF_AVErage {OFF|ON|0|1}
        - SELect:RF_AVErage?

    **Info:**
        - ``OFF`` or 0 turns the RF Average trace display off.
        - ``ON`` or 1 turns it on.
    """


class SelectRefItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:REF<x>`` command.

    **Description:**
        - Turns on and off the display of the reference waveform <x>. The <x > variable represents
          the reference channel number, which can be 1-4. The query returns whether the channel is
          on or off.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:REF<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:REF<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:REF<x> value`` command.

    **SCPI Syntax:**

    ::

        - SELect:REF<x> {ON|OFF|<NR1>}
        - SELect:REF<x>?

    **Info:**
        - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``OFF`` turns off the display of the specified waveform.
        - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns on
          the display of the specified waveform.
    """


class SelectMath1(SCPICmdWrite, SCPICmdRead):
    """The ``SELect:MATH1`` command.

    **Description:**
        - Turns on and off the display of the math waveform. The query returns whether the math
          waveform is on or off but does not indicate whether it is the selected waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:MATH1?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:MATH1?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:MATH1 value`` command.

    **SCPI Syntax:**

    ::

        - SELect:MATH1 {ON|OFF|<NR1>}
        - SELect:MATH1?

    **Info:**
        - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``OFF`` turns off the display of the specified waveform.
        - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns on
          the display of the specified waveform.
    """


class SelectMath(SCPICmdWrite, SCPICmdRead):
    """The ``SELect:MATH`` command.

    **Description:**
        - Turns on and off the display of the math waveform. The query returns whether the math
          waveform is on or off but does not indicate whether it is the selected waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:MATH?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:MATH?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:MATH value`` command.

    **SCPI Syntax:**

    ::

        - SELect:MATH {ON|OFF|<NR1>}
        - SELect:MATH?

    **Info:**
        - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``OFF`` turns off the display of the specified waveform.
        - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns on
          the display of the specified waveform.
    """


class SelectDall(SCPICmdWrite):
    """The ``SELect:DAll`` command.

    **Description:**
        - This command turns on or off all digital channels (D0 - D15).

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SELect:DAll value`` command.

    **SCPI Syntax:**

    ::

        - SELect:DAll {ON|OFF|0|1}
    """


class SelectDigitalBit(ValidatedDigitalBit, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:D<x>`` command.

    **Description:**
        - Turns on the display of the digital channel <x> and resets the acquisition. <x > is the
          channel number, which can be 0-15. The query returns whether the channel is on or off but
          does not indicate whether it is the selected waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:D<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:D<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:D<x> value`` command.

    **SCPI Syntax:**

    ::

        - SELect:D<x> {<NR1>|OFF|ON}
        - SELect:D<x>?

    **Info:**
        - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``OFF`` turns off the display of the specified waveform.
        - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns on
          the display of the specified waveform.
    """


class SelectControl(SCPICmdWrite, SCPICmdRead):
    """The ``SELect:CONTROl`` command.

    **Description:**
        - This command selects a waveform to be used with channel-related commands such as the
          cursor commands. In addition to selecting the waveform, it turns on the display of the
          waveform, if it wasn't on already.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:CONTROl?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:CONTROl?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:CONTROl value`` command.

    **SCPI Syntax:**

    ::

        - SELect:CONTROl {CH<x> |MATH|REF<x>|BUS<x>|D<x>|RF_AMPlitude|RF_FREQuency |RF_PHASe|RF_NORMal|RF_AVErage| RF_MAXHold|RF_MINHold}
        - SELect:CONTROl?

    **Info:**
        - ``CH<x>`` specifies a channel waveform as the waveform affected by the front-panel
          controls. x has a minimum of 1 and a maximum of 4.
        - ``MATH`` specifies the math waveform as the waveform that is affected by the front-panel
          controls.
        - ``BUS<x>`` specifies a bus waveform as the waveform affected by the front-panel controls.
          x has a minimum of 1 and a maximum of 4.
        - ``D<x>`` specifies a digital waveform as the waveform affected by the front-panel
          controls. (Requires option 3-MSO.) x has a minimum of 0 and a maximum of 15.
        - ``RF_NORMal`` specify an RF trace as the waveform affected by the front-panel controls.
        - ``RF_AVErage`` specify an RF trace as the waveform affected by the front-panel controls.
        - ``RF_MAXHold`` specify an RF trace as the waveform affected by the front-panel controls.
        - ``RF_MINHold`` specify an RF trace as the waveform affected by the front-panel controls.
    """  # noqa: E501


class SelectChannel(ValidatedChannel, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:CH<x>`` command.

    **Description:**
        - Turns the display of the channel <x> waveform on or off, where <x > is the channel number,
          which can be 1-4. This command also resets the acquisition. The query returns whether the
          channel is on or off but does not indicate whether it is the specified waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:CH<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:CH<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:CH<x> value`` command.

    **SCPI Syntax:**

    ::

        - SELect:CH<x> {ON|OFF|1|0}
        - SELect:CH<x>?

    **Info:**
        - ``ON`` or 1 turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``OFF`` or 0 turns off the display of the specified waveform.
    """


class SelectBusItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:BUS<x>`` command.

    **Description:**
        - This command turns on and off the display of the waveform for <x>, where x is the bus
          number 1-2. The query returns whether the channel is on or off but does not indicate
          whether it is the specified waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:BUS<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:BUS<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:BUS<x> value`` command.

    **SCPI Syntax:**

    ::

        - SELect:BUS<x> {OFF|ON|0|1}
        - SELect:BUS<x>?

    **Info:**
        - ``ON`` or 1 turns the specified waveform display on.
        - ``OFF`` or 0 turns the specified waveform display off.
    """


class SelectBItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:B<x>`` command.

    **Description:**
        - This command turns on and off the display of the waveform for <x>, where x is the bus
          number 1-2. The query returns whether the channel is on or off but does not indicate
          whether it is the specified waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:B<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:B<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:B<x> value`` command.

    **SCPI Syntax:**

    ::

        - SELect:B<x> {OFF|ON|0|1}
        - SELect:B<x>?

    **Info:**
        - ``ON`` or 1 turns the specified waveform display on.
        - ``OFF`` or 0 turns the specified waveform display off.
    """


#  pylint: disable=too-many-instance-attributes
class Select(SCPICmdRead):
    """The ``SELect`` command.

    **Description:**
        - Queries which waveforms are displayed.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SELect?

    Properties:
        - ``.ch``: The ``SELect:CH<x>`` command.
        - ``.control``: The ``SELect:CONTROl`` command.
        - ``.d``: The ``SELect:D<x>`` command.
        - ``.dall``: The ``SELect:DAll`` command.
        - ``.ref``: The ``SELect:REF<x>`` command.
        - ``.rf_average``: The ``SELect:RF_AVErage`` command.
        - ``.rf_maxhold``: The ``SELect:RF_MAXHold`` command.
        - ``.rf_minhold``: The ``SELect:RF_MINHold`` command.
        - ``.rf_normal``: The ``SELect:RF_NORMal`` command.
        - ``.bus``: The ``SELect:BUS<x>`` command.
        - ``.b``: The ``SELect:B<x>`` command.
        - ``.math``: The ``SELect:MATH`` command.
        - ``.math1``: The ``SELect:MATH1`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "SELect") -> None:
        super().__init__(device, cmd_syntax)
        self._ch: Dict[int, SelectChannel] = DefaultDictPassKeyToFactory(
            lambda x: SelectChannel(device, f"{self._cmd_syntax}:CH{x}")
        )
        self._control = SelectControl(device, f"{self._cmd_syntax}:CONTROl")
        self._d: Dict[int, SelectDigitalBit] = DefaultDictPassKeyToFactory(
            lambda x: SelectDigitalBit(device, f"{self._cmd_syntax}:D{x}")
        )
        self._dall = SelectDall(device, f"{self._cmd_syntax}:DAll")
        self._ref: Dict[int, SelectRefItem] = DefaultDictPassKeyToFactory(
            lambda x: SelectRefItem(device, f"{self._cmd_syntax}:REF{x}")
        )
        self._rf_average = SelectRfAverage(device, f"{self._cmd_syntax}:RF_AVErage")
        self._rf_maxhold = SelectRfMaxhold(device, f"{self._cmd_syntax}:RF_MAXHold")
        self._rf_minhold = SelectRfMinhold(device, f"{self._cmd_syntax}:RF_MINHold")
        self._rf_normal = SelectRfNormal(device, f"{self._cmd_syntax}:RF_NORMal")
        self._bus: Dict[int, SelectBusItem] = DefaultDictPassKeyToFactory(
            lambda x: SelectBusItem(device, f"{self._cmd_syntax}:BUS{x}")
        )
        self._b: Dict[int, SelectBItem] = DefaultDictPassKeyToFactory(
            lambda x: SelectBItem(device, f"{self._cmd_syntax}:B{x}")
        )
        self._math = SelectMath(device, f"{self._cmd_syntax}:MATH")
        self._math1 = SelectMath1(device, f"{self._cmd_syntax}:MATH1")

    @property
    def ch(self) -> Dict[int, SelectChannel]:
        """Return the ``SELect:CH<x>`` command.

        **Description:**
            - Turns the display of the channel <x> waveform on or off, where <x > is the channel
              number, which can be 1-4. This command also resets the acquisition. The query returns
              whether the channel is on or off but does not indicate whether it is the specified
              waveform.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:CH<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:CH<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:CH<x> value`` command.

        **SCPI Syntax:**

        ::

            - SELect:CH<x> {ON|OFF|1|0}
            - SELect:CH<x>?

        **Info:**
            - ``ON`` or 1 turns on the display of the specified waveform. This waveform also becomes
              the selected waveform.
            - ``OFF`` or 0 turns off the display of the specified waveform.
        """
        return self._ch

    @property
    def control(self) -> SelectControl:
        """Return the ``SELect:CONTROl`` command.

        **Description:**
            - This command selects a waveform to be used with channel-related commands such as the
              cursor commands. In addition to selecting the waveform, it turns on the display of the
              waveform, if it wasn't on already.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:CONTROl?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:CONTROl?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:CONTROl value`` command.

        **SCPI Syntax:**

        ::

            - SELect:CONTROl {CH<x> |MATH|REF<x>|BUS<x>|D<x>|RF_AMPlitude|RF_FREQuency |RF_PHASe|RF_NORMal|RF_AVErage| RF_MAXHold|RF_MINHold}
            - SELect:CONTROl?

        **Info:**
            - ``CH<x>`` specifies a channel waveform as the waveform affected by the front-panel
              controls. x has a minimum of 1 and a maximum of 4.
            - ``MATH`` specifies the math waveform as the waveform that is affected by the
              front-panel controls.
            - ``BUS<x>`` specifies a bus waveform as the waveform affected by the front-panel
              controls. x has a minimum of 1 and a maximum of 4.
            - ``D<x>`` specifies a digital waveform as the waveform affected by the front-panel
              controls. (Requires option 3-MSO.) x has a minimum of 0 and a maximum of 15.
            - ``RF_NORMal`` specify an RF trace as the waveform affected by the front-panel
              controls.
            - ``RF_AVErage`` specify an RF trace as the waveform affected by the front-panel
              controls.
            - ``RF_MAXHold`` specify an RF trace as the waveform affected by the front-panel
              controls.
            - ``RF_MINHold`` specify an RF trace as the waveform affected by the front-panel
              controls.
        """  # noqa: E501
        return self._control

    @property
    def d(self) -> Dict[int, SelectDigitalBit]:
        """Return the ``SELect:D<x>`` command.

        **Description:**
            - Turns on the display of the digital channel <x> and resets the acquisition. <x > is
              the channel number, which can be 0-15. The query returns whether the channel is on or
              off but does not indicate whether it is the selected waveform.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:D<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:D<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:D<x> value`` command.

        **SCPI Syntax:**

        ::

            - SELect:D<x> {<NR1>|OFF|ON}
            - SELect:D<x>?

        **Info:**
            - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``OFF`` turns off the display of the specified waveform.
            - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns
              on the display of the specified waveform.
        """
        return self._d

    @property
    def dall(self) -> SelectDall:
        """Return the ``SELect:DAll`` command.

        **Description:**
            - This command turns on or off all digital channels (D0 - D15).

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SELect:DAll value`` command.

        **SCPI Syntax:**

        ::

            - SELect:DAll {ON|OFF|0|1}
        """
        return self._dall

    @property
    def ref(self) -> Dict[int, SelectRefItem]:
        """Return the ``SELect:REF<x>`` command.

        **Description:**
            - Turns on and off the display of the reference waveform <x>. The <x > variable
              represents the reference channel number, which can be 1-4. The query returns whether
              the channel is on or off.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:REF<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:REF<x>?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:REF<x> value`` command.

        **SCPI Syntax:**

        ::

            - SELect:REF<x> {ON|OFF|<NR1>}
            - SELect:REF<x>?

        **Info:**
            - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``OFF`` turns off the display of the specified waveform.
            - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns
              on the display of the specified waveform.
        """
        return self._ref

    @property
    def rf_average(self) -> SelectRfAverage:
        """Return the ``SELect:RF_AVErage`` command.

        **Description:**
            - This command switches the RF Average trace display on or off in the frequency domain
              graticule.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:RF_AVErage?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:RF_AVErage?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:RF_AVErage value`` command.

        **SCPI Syntax:**

        ::

            - SELect:RF_AVErage {OFF|ON|0|1}
            - SELect:RF_AVErage?

        **Info:**
            - ``OFF`` or 0 turns the RF Average trace display off.
            - ``ON`` or 1 turns it on.
        """
        return self._rf_average

    @property
    def rf_maxhold(self) -> SelectRfMaxhold:
        """Return the ``SELect:RF_MAXHold`` command.

        **Description:**
            - This command switches the frequency domain Max Hold trace display on or off in the
              frequency domain graticule.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:RF_MAXHold?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:RF_MAXHold?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:RF_MAXHold value`` command.

        **SCPI Syntax:**

        ::

            - SELect:RF_MAXHold {OFF|ON|0|1}
            - SELect:RF_MAXHold?

        **Info:**
            - ``OFF`` or 0 turns the frequency domain Max Hold trace display off.
            - ``ON`` or 1 turns it on.
        """
        return self._rf_maxhold

    @property
    def rf_minhold(self) -> SelectRfMinhold:
        """Return the ``SELect:RF_MINHold`` command.

        **Description:**
            - This command switches the frequency domain Min Hold trace display on or off in the
              frequency domain graticule.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:RF_MINHold?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:RF_MINHold?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:RF_MINHold value`` command.

        **SCPI Syntax:**

        ::

            - SELect:RF_MINHold {OFF|ON|0|1}
            - SELect:RF_MINHold?

        **Info:**
            - ``OFF`` or 0 turns the frequency domain Min Hold display off.
            - ``ON`` or 1 turns it on.
        """
        return self._rf_minhold

    @property
    def rf_normal(self) -> SelectRfNormal:
        """Return the ``SELect:RF_NORMal`` command.

        **Description:**
            - This command switches the frequency domain Normal trace display on or off in the
              frequency domain graticule.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:RF_NORMal?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:RF_NORMal?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:RF_NORMal value`` command.

        **SCPI Syntax:**

        ::

            - SELect:RF_NORMal {OFF|ON|0|1}
            - SELect:RF_NORMal?

        **Info:**
            - ``OFF`` or 0 turns the frequency domain Normal trace display off.
            - ``ON`` or 1 turns it on.
        """
        return self._rf_normal

    @property
    def bus(self) -> Dict[int, SelectBusItem]:
        """Return the ``SELect:BUS<x>`` command.

        **Description:**
            - This command turns on and off the display of the waveform for <x>, where x is the bus
              number 1-2. The query returns whether the channel is on or off but does not indicate
              whether it is the specified waveform.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:BUS<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:BUS<x>?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:BUS<x> value`` command.

        **SCPI Syntax:**

        ::

            - SELect:BUS<x> {OFF|ON|0|1}
            - SELect:BUS<x>?

        **Info:**
            - ``ON`` or 1 turns the specified waveform display on.
            - ``OFF`` or 0 turns the specified waveform display off.
        """
        return self._bus

    @property
    def b(self) -> Dict[int, SelectBItem]:
        """Return the ``SELect:B<x>`` command.

        **Description:**
            - This command turns on and off the display of the waveform for <x>, where x is the bus
              number 1-2. The query returns whether the channel is on or off but does not indicate
              whether it is the specified waveform.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:B<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:B<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:B<x> value`` command.

        **SCPI Syntax:**

        ::

            - SELect:B<x> {OFF|ON|0|1}
            - SELect:B<x>?

        **Info:**
            - ``ON`` or 1 turns the specified waveform display on.
            - ``OFF`` or 0 turns the specified waveform display off.
        """
        return self._b

    @property
    def math(self) -> SelectMath:
        """Return the ``SELect:MATH`` command.

        **Description:**
            - Turns on and off the display of the math waveform. The query returns whether the math
              waveform is on or off but does not indicate whether it is the selected waveform.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:MATH?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:MATH?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:MATH value`` command.

        **SCPI Syntax:**

        ::

            - SELect:MATH {ON|OFF|<NR1>}
            - SELect:MATH?

        **Info:**
            - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``OFF`` turns off the display of the specified waveform.
            - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns
              on the display of the specified waveform.
        """
        return self._math

    @property
    def math1(self) -> SelectMath1:
        """Return the ``SELect:MATH1`` command.

        **Description:**
            - Turns on and off the display of the math waveform. The query returns whether the math
              waveform is on or off but does not indicate whether it is the selected waveform.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:MATH1?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:MATH1?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:MATH1 value`` command.

        **SCPI Syntax:**

        ::

            - SELect:MATH1 {ON|OFF|<NR1>}
            - SELect:MATH1?

        **Info:**
            - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``OFF`` turns off the display of the specified waveform.
            - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns
              on the display of the specified waveform.
        """
        return self._math1



# pylint: disable=too-many-lines
# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The channel commands module.

These commands are used in the following models:
SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - channel.calibration.adjustcount()
    - channel.calibration.adjustdate()
    - channel.calibration.lock()
    - channel.calibration.password()
    - channel.calibration.save()
    - channel.calibration.step()
    - channel.calibration.unlock()
    - channel.calibration.verifydate()
    - channel.clearforbidden()
    - channel.close()
    - channel.connectrule
    - channel.connectsequential
    - channel.exclusiveclose()
    - channel.exclusiveslotclose()
    - channel.getbackplane()
    - channel.getclose()
    - channel.getcount()
    - channel.getdelay()
    - channel.getforbidden()
    - channel.getimage()
    - channel.getlabel()
    - channel.getmatch()
    - channel.getmatchtype()
    - channel.getmode()
    - channel.getoutputenable()
    - channel.getpole()
    - channel.getpowerstate()
    - channel.getstate()
    - channel.getstatelatch()
    - channel.gettype()
    - channel.open()
    - channel.pattern.delete()
    - channel.pattern.getimage()
    - channel.pattern.setimage()
    - channel.pattern.snapshot()
    - channel.read()
    - channel.reset()
    - channel.resetstatelatch()
    - channel.setbackplane()
    - channel.setdelay()
    - channel.setforbidden()
    - channel.setlabel()
    - channel.setmatch()
    - channel.setmatchtype()
    - channel.setmode()
    - channel.setoutputenable()
    - channel.setpole()
    - channel.setstatelatch()
    - channel.trigger[N].clear()
    - channel.trigger[N].get()
    - channel.trigger[N].set()
    - channel.trigger[N].wait()
    - channel.write()
"""
from typing import Dict, Optional, TYPE_CHECKING, Union

from .._helpers import (
    BaseTSPCmd,
    DefaultDictPassKeyToFactory,
    NoDeviceProvidedError,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class ChannelTriggerItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``channel.trigger[N]`` command tree.

    **Info:**
        - ``N``, the number indicating the channel trigger event ID (1 to 8).

    Constants:
        - ``.EVENT_ID``: This constant indicates the trigger event generated by the channel trigger
          N.

    Properties/methods:
        - ``.clear()``: The ``channel.trigger[N].clear()`` function.
        - ``.get()``: The ``channel.trigger[N].get()`` function.
        - ``.set()``: The ``channel.trigger[N].set()`` function.
        - ``.wait()``: The ``channel.trigger[N].wait()`` function.
    """

    EVENT_ID = "channel.trigger[N].EVENT_ID"
    """str: This constant indicates the trigger event generated by the channel trigger N."""

    def __init__(self, device: Optional["TSPDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        # pylint: disable=invalid-name
        self.EVENT_ID = self.EVENT_ID.replace(
            "[N]", f"[{self._cmd_syntax.rsplit('[', maxsplit=1)[-1].split(']', maxsplit=1)[0]}]"
        )

    def clear(self) -> None:
        """Run the ``channel.trigger[N].clear()`` function.

        **Description:**
            - This function clears any pending events.

        **TSP Syntax:**

        ::

            - channel.trigger[N].clear()

        **Info:**
            - ``N``, the number indicating the trigger line to clear (1 to 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clear()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def get(self) -> str:
        """Run the ``channel.trigger[N].get()`` function.

        **Description:**
            - This function gets the channel status trigger information that is used to watch the
              state of a specific channel.

        **TSP Syntax:**

        ::

            - channel.trigger[N].get()

        **Info:**
            - ``N``, the number indicating the channel trigger to get (1 to 8).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.get())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.get()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def set_(self, channel_list: str, state_match: str) -> None:
        """Run the ``channel.trigger[N].set()`` function.

        **Description:**
            - This function sets the channel status trigger model to watch the state of a specific
              channel.

        **TSP Syntax:**

        ::

            - channel.trigger[N].set()

        Args:
            channel_list: The channels to set, using standard channel naming.
            state_match: Value specifying the status to match when triggering an event.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.set("{channel_list}", {state_match})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.set()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def wait(self, timeout: float) -> str:
        """Run the ``channel.trigger[N].wait()`` function.

        **Description:**
            - This function waits for the specified trigger or timeout period, whichever comes
              first.

        **TSP Syntax:**

        ::

            - channel.trigger[N].wait()

        Args:
            timeout: The number of seconds to wait.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.wait({timeout}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.wait()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class ChannelPattern(BaseTSPCmd):
    """The ``channel.pattern`` command tree.

    Properties/methods:
        - ``.delete()``: The ``channel.pattern.delete()`` function.
        - ``.getimage()``: The ``channel.pattern.getimage()`` function.
        - ``.setimage()``: The ``channel.pattern.setimage()`` function.
        - ``.snapshot()``: The ``channel.pattern.snapshot()`` function.
    """

    def delete(self, name: str) -> None:
        """Run the ``channel.pattern.delete()`` function.

        **Description:**
            - This function deletes a channel pattern.

        **TSP Syntax:**

        ::

            - channel.pattern.delete()

        Args:
            name: A string representing the name of the channel pattern to delete.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f'{self._cmd_syntax}.delete("{name}")')  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.delete()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getimage(self, name: str) -> str:
        """Run the ``channel.pattern.getimage()`` function.

        **Description:**
            - This function queries a channel pattern for associated channels and analog backplane
              relays.

        **TSP Syntax:**

        ::

            - channel.pattern.getimage()

        Args:
            name: A string representing the name of the channel pattern to query.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getimage("{name}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getimage()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setimage(self, channel_list: str, name: str) -> None:
        """Run the ``channel.pattern.setimage()`` function.

        **Description:**
            - This function creates a channel pattern and associates it with the specified name.

        **TSP Syntax:**

        ::

            - channel.pattern.setimage()

        Args:
            channel_list: A string listing the channels, channel patterns, or analog backplane
                relays to use when creating the new channel pattern.
            name: A string representing the name to associate with the new channel pattern.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setimage("{channel_list}", "{name}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setimage()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def snapshot(self, name: str) -> None:
        """Run the ``channel.pattern.snapshot()`` function.

        **Description:**
            - This function creates a channel pattern.

        **TSP Syntax:**

        ::

            - channel.pattern.snapshot()

        Args:
            name: A string representing the name to associate with the present state of channels and
                analog backplane relays.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f'{self._cmd_syntax}.snapshot("{name}")')  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.snapshot()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class ChannelCalibration(BaseTSPCmd):
    """The ``channel.calibration`` command tree.

    Properties/methods:
        - ``.adjustcount()``: The ``channel.calibration.adjustcount()`` function.
        - ``.adjustdate()``: The ``channel.calibration.adjustdate()`` function.
        - ``.lock()``: The ``channel.calibration.lock()`` function.
        - ``.password()``: The ``channel.calibration.password()`` function.
        - ``.save()``: The ``channel.calibration.save()`` function.
        - ``.step()``: The ``channel.calibration.step()`` function.
        - ``.unlock()``: The ``channel.calibration.unlock()`` function.
        - ``.verifydate()``: The ``channel.calibration.verifydate()`` function.
    """

    def adjustcount(self, x: str) -> str:
        """Run the ``channel.calibration.adjustcount()`` function.

        **Description:**
            - This function gets the number of times that a card has been adjusted.

        **TSP Syntax:**

        ::

            - channel.calibration.adjustcount()

        Args:
            x: The slot number ( 1 to 6).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.adjustcount({x}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.adjustcount()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def adjustdate(self, x: str, date: Optional[str] = None) -> str:
        """Run the ``channel.calibration.adjustdate()`` function.

        **Description:**
            - This function sets or gets the adjustment date in UTC format (number of seconds since
              January 1, 1970) on the unlocked channel.

        **TSP Syntax:**

        ::

            - channel.calibration.adjustdate()

        Args:
            x: The slot number ( 1 to 6).
            date (optional): The date of the adjustment (UTC formatted).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    x,
                    date,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.adjustdate({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.adjustdate()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def lock(self) -> None:
        """Run the ``channel.calibration.lock()`` function.

        **Description:**
            - This function prevents further calibration on the currently unlocked card.

        **TSP Syntax:**

        ::

            - channel.calibration.lock()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.lock()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.lock()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def password(self, password: str) -> None:
        """Run the ``channel.calibration.password()`` function.

        **Description:**
            - This function sets the password needed to unlock the calibration functionality of a
              card.

        **TSP Syntax:**

        ::

            - channel.calibration.password()

        Args:
            password: A string of characters that contain the password.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.password("{password}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.password()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def save(self) -> None:
        """Run the ``channel.calibration.save()`` function.

        **Description:**
            - This function saves the calibration data to the card.

        **TSP Syntax:**

        ::

            - channel.calibration.save()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.save()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.save()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def step(self, channel: str, step: str, value: Optional[str] = None) -> None:
        """Run the ``channel.calibration.step()`` function.

        **Description:**
            - This function sends a calibration command.

        **TSP Syntax:**

        ::

            - channel.calibration.step()

        Args:
            channel: The channel to be calibrated.
            step: The number corresponding to the specified step.
            value (optional): The measured value for the specified step when the step value is even.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    channel,
                    step,
                    value,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.step({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.step()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def unlock(self, x: str, password: str) -> None:
        """Run the ``channel.calibration.unlock()`` function.

        **Description:**
            - This function unlocks calibration functionality for a card so that calibration
              operations can be performed.

        **TSP Syntax:**

        ::

            - channel.calibration.unlock()

        Args:
            x: The slot number ( 1 to 6).
            password: The password that unlocks calibration.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.unlock({x}, {password})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.unlock()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def verifydate(self, x: str, date: Optional[str] = None) -> str:
        """Run the ``channel.calibration.verifydate()`` function.

        **Description:**
            - This function gets or sets the date the calibration was verified in UTC format (number
              of seconds since January 1, 1970).

        **TSP Syntax:**

        ::

            - channel.calibration.verifydate()

        Args:
            x: The slot number ( 1 to 6).
            date (optional): UTC formatted date to which to set the calibration verification date.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    x,
                    date,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.verifydate({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.verifydate()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


#  pylint: disable=too-many-public-methods
class Channel(BaseTSPCmd):
    """The ``channel`` command tree.

    Properties/methods:
        - ``.calibration``: The ``channel.calibration`` command tree.
        - ``.clearforbidden()``: The ``channel.clearforbidden()`` function.
        - ``.close()``: The ``channel.close()`` function.
        - ``.connectrule``: The ``channel.connectrule`` attribute.
        - ``.connectsequential``: The ``channel.connectsequential`` attribute.
        - ``.exclusiveclose()``: The ``channel.exclusiveclose()`` function.
        - ``.exclusiveslotclose()``: The ``channel.exclusiveslotclose()`` function.
        - ``.getbackplane()``: The ``channel.getbackplane()`` function.
        - ``.getclose()``: The ``channel.getclose()`` function.
        - ``.getcount()``: The ``channel.getcount()`` function.
        - ``.getdelay()``: The ``channel.getdelay()`` function.
        - ``.getforbidden()``: The ``channel.getforbidden()`` function.
        - ``.getimage()``: The ``channel.getimage()`` function.
        - ``.getlabel()``: The ``channel.getlabel()`` function.
        - ``.getmatch()``: The ``channel.getmatch()`` function.
        - ``.getmatchtype()``: The ``channel.getmatchtype()`` function.
        - ``.getmode()``: The ``channel.getmode()`` function.
        - ``.getoutputenable()``: The ``channel.getoutputenable()`` function.
        - ``.getpole()``: The ``channel.getpole()`` function.
        - ``.getpowerstate()``: The ``channel.getpowerstate()`` function.
        - ``.getstate()``: The ``channel.getstate()`` function.
        - ``.getstatelatch()``: The ``channel.getstatelatch()`` function.
        - ``.gettype()``: The ``channel.gettype()`` function.
        - ``.open()``: The ``channel.open()`` function.
        - ``.pattern``: The ``channel.pattern`` command tree.
        - ``.read()``: The ``channel.read()`` function.
        - ``.reset()``: The ``channel.reset()`` function.
        - ``.resetstatelatch()``: The ``channel.resetstatelatch()`` function.
        - ``.setbackplane()``: The ``channel.setbackplane()`` function.
        - ``.setdelay()``: The ``channel.setdelay()`` function.
        - ``.setforbidden()``: The ``channel.setforbidden()`` function.
        - ``.setlabel()``: The ``channel.setlabel()`` function.
        - ``.setmatch()``: The ``channel.setmatch()`` function.
        - ``.setmatchtype()``: The ``channel.setmatchtype()`` function.
        - ``.setmode()``: The ``channel.setmode()`` function.
        - ``.setoutputenable()``: The ``channel.setoutputenable()`` function.
        - ``.setpole()``: The ``channel.setpole()`` function.
        - ``.setstatelatch()``: The ``channel.setstatelatch()`` function.
        - ``.trigger``: The ``channel.trigger[N]`` command tree.
        - ``.write()``: The ``channel.write()`` function.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "channel") -> None:
        super().__init__(device, cmd_syntax)
        self._calibration = ChannelCalibration(device, f"{self._cmd_syntax}.calibration")
        self._pattern = ChannelPattern(device, f"{self._cmd_syntax}.pattern")
        self._trigger: Dict[int, ChannelTriggerItem] = DefaultDictPassKeyToFactory(
            lambda x: ChannelTriggerItem(device, f"{self._cmd_syntax}.trigger[{x}]")
        )

    @property
    def calibration(self) -> ChannelCalibration:
        """Return the ``channel.calibration`` command tree.

        Sub-properties/methods:
            - ``.adjustcount()``: The ``channel.calibration.adjustcount()`` function.
            - ``.adjustdate()``: The ``channel.calibration.adjustdate()`` function.
            - ``.lock()``: The ``channel.calibration.lock()`` function.
            - ``.password()``: The ``channel.calibration.password()`` function.
            - ``.save()``: The ``channel.calibration.save()`` function.
            - ``.step()``: The ``channel.calibration.step()`` function.
            - ``.unlock()``: The ``channel.calibration.unlock()`` function.
            - ``.verifydate()``: The ``channel.calibration.verifydate()`` function.
        """
        return self._calibration

    @property
    def connectrule(self) -> str:
        """Access the ``channel.connectrule`` attribute.

        **Description:**
            - This attribute controls the connection rule for closing and opening channels in the
              instrument.

        **Usage:**
            - Accessing this property will send the ``print(channel.connectrule)`` query.
            - Setting this property to a value will send the ``channel.connectrule = value``
              command.

        **TSP Syntax:**

        ::

            - channel.connectrule = value
            - print(channel.connectrule)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".connectrule"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.connectrule)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.connectrule`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @connectrule.setter
    def connectrule(self, value: Union[str, float]) -> None:
        """Access the ``channel.connectrule`` attribute.

        **Description:**
            - This attribute controls the connection rule for closing and opening channels in the
              instrument.

        **Usage:**
            - Accessing this property will send the ``print(channel.connectrule)`` query.
            - Setting this property to a value will send the ``channel.connectrule = value``
              command.

        **TSP Syntax:**

        ::

            - channel.connectrule = value
            - print(channel.connectrule)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".connectrule", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.connectrule = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.connectrule`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def connectsequential(self) -> str:
        """Access the ``channel.connectsequential`` attribute.

        **Description:**
            - This attribute controls whether or not channels are closed sequentially.

        **Usage:**
            - Accessing this property will send the ``print(channel.connectsequential)`` query.
            - Setting this property to a value will send the ``channel.connectsequential = value``
              command.

        **TSP Syntax:**

        ::

            - channel.connectsequential = value
            - print(channel.connectsequential)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".connectsequential"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.connectsequential)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.connectsequential`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @connectsequential.setter
    def connectsequential(self, value: Union[str, float]) -> None:
        """Access the ``channel.connectsequential`` attribute.

        **Description:**
            - This attribute controls whether or not channels are closed sequentially.

        **Usage:**
            - Accessing this property will send the ``print(channel.connectsequential)`` query.
            - Setting this property to a value will send the ``channel.connectsequential = value``
              command.

        **TSP Syntax:**

        ::

            - channel.connectsequential = value
            - print(channel.connectsequential)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".connectsequential", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.connectsequential = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.connectsequential`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def pattern(self) -> ChannelPattern:
        """Return the ``channel.pattern`` command tree.

        Sub-properties/methods:
            - ``.delete()``: The ``channel.pattern.delete()`` function.
            - ``.getimage()``: The ``channel.pattern.getimage()`` function.
            - ``.setimage()``: The ``channel.pattern.setimage()`` function.
            - ``.snapshot()``: The ``channel.pattern.snapshot()`` function.
        """
        return self._pattern

    @property
    def trigger(self) -> Dict[int, ChannelTriggerItem]:
        """Return the ``channel.trigger[N]`` command tree.

        **Info:**
            - ``N``, the number indicating the channel trigger event ID (1 to 8).

        Constants:
            - ``.EVENT_ID``: This constant indicates the trigger event generated by the channel
              trigger N.

        Sub-properties/methods:
            - ``.clear()``: The ``channel.trigger[N].clear()`` function.
            - ``.get()``: The ``channel.trigger[N].get()`` function.
            - ``.set()``: The ``channel.trigger[N].set()`` function.
            - ``.wait()``: The ``channel.trigger[N].wait()`` function.
        """
        return self._trigger

    def clearforbidden(self, channel_list: str) -> None:
        """Run the ``channel.clearforbidden()`` function.

        **Description:**
            - This function clears the list of channels specified from being forbidden to close.

        **TSP Syntax:**

        ::

            - channel.clearforbidden()

        Args:
            channel_list: The channels to set, using standard channel naming.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.clearforbidden("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clearforbidden()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def close(self, channel_list: str) -> None:
        """Run the ``channel.close()`` function.

        **Description:**
            - This function closes the channels, analog backplane relays, and channel patterns that
              are specified by the channel list parameter.

        **TSP Syntax:**

        ::

            - channel.close()

        Args:
            channel_list: The channels, analog backplane relays, and channel patterns to close,
                using standard channel naming.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.close("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.close()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def exclusiveclose(self, channel_list: str) -> None:
        """Run the ``channel.exclusiveclose()`` function.

        **Description:**
            - This function closes the specified channels so that they are the only channels that
              are closed on the instrument.

        **TSP Syntax:**

        ::

            - channel.exclusiveclose()

        Args:
            channel_list: A string listing the channels (including analog backplane relays) and
                channel patterns to exclusively close.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.exclusiveclose("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.exclusiveclose()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def exclusiveslotclose(self, channel_list: str) -> None:
        """Run the ``channel.exclusiveslotclose()`` function.

        **Description:**
            - This function closes the specified channels and channel patterns on the associated
              slots and opens any channels that are not specified.

        **TSP Syntax:**

        ::

            - channel.exclusiveslotclose()

        Args:
            channel_list: A string that lists the channels and channel patterns to exclusively close
                on the cards in associated slots (you can specify analog backplane relays).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.exclusiveslotclose("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.exclusiveslotclose()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getbackplane(self, channel_list: str) -> str:
        """Run the ``channel.getbackplane()`` function.

        **Description:**
            - This function returns a string that lists the analog backplane relays that are
              controlled when the specified channels are used with switching operations.

        **TSP Syntax:**

        ::

            - channel.getbackplane()

        Args:
            channel_list: A string listing the channels being queried.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getbackplane("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getbackplane()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getclose(self, channel_list: str) -> str:
        """Run the ``channel.getclose()`` function.

        **Description:**
            - This function queries for the closed channels indicated by the channel list parameter.

        **TSP Syntax:**

        ::

            - channel.getclose()

        Args:
            channel_list: A string representing the channels, channel patterns, and backplane relays
                that will be queried.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getclose("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getclose()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getcount(self, channel_list: str) -> str:
        """Run the ``channel.getcount()`` function.

        **Description:**
            - This function returns the number of times the relays have been closed for the
              specified channels.

        **TSP Syntax:**

        ::

            - channel.getcount()

        Args:
            channel_list: A string listing the items to query, which can include.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getcount("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getcount()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getdelay(self, channel_list: str) -> str:
        """Run the ``channel.getdelay()`` function.

        **Description:**
            - This function queries for the additional delay time for the specified channels.

        **TSP Syntax:**

        ::

            - channel.getdelay()

        Args:
            channel_list: A string listing the channels to query for their delay times.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getdelay("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getdelay()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getforbidden(self, channel_list: str) -> str:
        """Run the ``channel.getforbidden()`` function.

        **Description:**
            - This function returns a string listing the channels and analog backplane relays in the
              channel list that are forbidden to close.

        **TSP Syntax:**

        ::

            - channel.getforbidden()

        Args:
            channel_list: A string listing the channels, backplane relays, and channel patterns that
                are to be checked to see if they are forbidden to close.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getforbidden("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getforbidden()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getimage(self, channel_list: str) -> str:
        """Run the ``channel.getimage()`` function.

        **Description:**
            - This function queries a channel for items associated with that channel when used in a
              switching operation.

        **TSP Syntax:**

        ::

            - channel.getimage()

        Args:
            channel_list: A string representing the channels and analog backplane relays to query.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getimage("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getimage()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getlabel(self, channel_list: str) -> str:
        """Run the ``channel.getlabel()`` function.

        **Description:**
            - This function retrieves the label associated with one or more channels.

        **TSP Syntax:**

        ::

            - channel.getlabel()

        Args:
            channel_list: A string listing the channels to query for the label associated with them.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getlabel("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getlabel()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getmatch(self, channel_list: str) -> str:
        """Run the ``channel.getmatch()`` function.

        **Description:**
            - This function gets the match value on a channel.

        **TSP Syntax:**

        ::

            - channel.getmatch()

        Args:
            channel_list: String specifying digital I/O or totalizer channels to query, using normal
                channel list syntax.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getmatch("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getmatch()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getmatchtype(self, channel_list: str) -> str:
        """Run the ``channel.getmatchtype()`` function.

        **Description:**
            - This function returns the match type for digital I/O and totalizer channels.

        **TSP Syntax:**

        ::

            - channel.getmatchtype()

        Args:
            channel_list: String specifying the digital I/O or totalizer channels to query, using
                normal channel list syntax.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getmatchtype("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getmatchtype()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getmode(self, channel_list: str) -> str:
        """Run the ``channel.getmode()`` function.

        **Description:**
            - This function returns the present mode of digital and totalizer channels.

        **TSP Syntax:**

        ::

            - channel.getmode()

        Args:
            channel_list: String that specifies the channels to query, using normal channel list
                syntax.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getmode("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getmode()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getoutputenable(self, channel_list: str) -> str:
        """Run the ``channel.getoutputenable()`` function.

        **Description:**
            - This function gets the present output enable attribute for a channel.

        **TSP Syntax:**

        ::

            - channel.getoutputenable()

        Args:
            channel_list: String specifying the DAC channels to query, using normal channel list
                syntax.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getoutputenable("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getoutputenable()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getpole(self, channel_list: str) -> str:
        """Run the ``channel.getpole()`` function.

        **Description:**
            - This function queries the pole settings for the specified channels.

        **TSP Syntax:**

        ::

            - channel.getpole()

        Args:
            channel_list: A string listing the channels to query for their pole settings.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getpole("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getpole()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getpowerstate(self, channel_list: str) -> str:
        """Run the ``channel.getpowerstate()`` function.

        **Description:**
            - This function gets the current power state attribute for a totalizer or DAC channel.

        **TSP Syntax:**

        ::

            - channel.getpowerstate()

        Args:
            channel_list: String specifying the channels to query, using normal channel list syntax.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getpowerstate("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getpowerstate()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getstate(self, channel_list: str, indicator_mask: Optional[str] = None) -> str:
        """Run the ``channel.getstate()`` function.

        **Description:**
            - This function returns the state indicators of the channels in the instrument.

        **TSP Syntax:**

        ::

            - channel.getstate()

        Args:
            channel_list: String specifying the channels to query, using normal channel list syntax.
            indicator_mask (optional): Value to specify only certain indicators; if omitted, all
                indicators are returned.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{channel_list}"',
                    indicator_mask,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.getstate({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getstate()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getstatelatch(self, channel_list: str) -> str:
        """Run the ``channel.getstatelatch()`` function.

        **Description:**
            - This function gets the mask representing the states that would be latched if they
              occurred.

        **TSP Syntax:**

        ::

            - channel.getstatelatch()

        Args:
            channel_list: String specifying the channels to query, using normal channel list syntax.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getstatelatch("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getstatelatch()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def gettype(self, channel_list: str) -> str:
        """Run the ``channel.gettype()`` function.

        **Description:**
            - This function returns the type associated with a channel.

        **TSP Syntax:**

        ::

            - channel.gettype()

        Args:
            channel_list: String specifying the channels to query, using normal channelList syntax.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.gettype("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.gettype()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def open(self, channel_list: str) -> None:
        """Run the ``channel.open()`` function.

        **Description:**
            - This function opens the specified channels, analog backplane relays, and channel
              patterns.

        **TSP Syntax:**

        ::

            - channel.open()

        Args:
            channel_list: String listing the channels, analog backplane relays, and channel patterns
                to open.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.open("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.open()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def read(
        self, channel_list: str, width: Optional[str] = None, reading_buffer: Optional[str] = None
    ) -> str:
        """Run the ``channel.read()`` function.

        **Description:**
            - This function reads a value from a totalizer, DAC, or digital I/O channel.

        **TSP Syntax:**

        ::

            - channel.read()

        Args:
            channel_list: The channels to set, using standard channel naming.
            width (optional): Specifies reading over multiple consecutive channels (default 1).
            reading_buffer (optional): Reading buffer to store read values.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{channel_list}"',
                    width,
                    reading_buffer,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.read({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.read()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def reset(self, channel_list: str) -> None:
        """Run the ``channel.reset()`` function.

        **Description:**
            - This function resets the specified channel list items to factory default settings.

        **TSP Syntax:**

        ::

            - channel.reset()

        Args:
            channel_list: A string that lists the items to reset; the string can include.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.reset("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.reset()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def resetstatelatch(self, channel_list: str, state: str) -> None:
        """Run the ``channel.resetstatelatch()`` function.

        **Description:**
            - This function resets the channel state.

        **TSP Syntax:**

        ::

            - channel.resetstatelatch()

        Args:
            channel_list: String that specifies the channels that need to have their states reset,
                using normal channel list syntax.
            state: The comma-delimited states for channels in channelList that are to have their
                states reset.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.resetstatelatch("{channel_list}", {state})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.resetstatelatch()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setbackplane(self, channel_list: str, abuslist: str) -> None:
        """Run the ``channel.setbackplane()`` function.

        **Description:**
            - This function specifies the list of analog backplane relays to use with the specified
              channels when they are used in switching applications.

        **TSP Syntax:**

        ::

            - channel.setbackplane()

        Args:
            channel_list: A string that lists the channels to change.
            abuslist: A string that lists the analog backplane relays to set for the channels
                specified in channelList.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setbackplane("{channel_list}", "{abuslist}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setbackplane()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setdelay(self, channel_list: str, delay: float) -> None:
        """Run the ``channel.setdelay()`` function.

        **Description:**
            - This function sets additional delay time for specified channels.

        **TSP Syntax:**

        ::

            - channel.setdelay()

        Args:
            channel_list: A string listing the channels for which to add delay time.
            delay: Delay time for the selected channels; minimum is 0 seconds.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setdelay("{channel_list}", {delay})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setdelay()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setforbidden(self, channel_list: str) -> None:
        """Run the ``channel.setforbidden()`` function.

        **Description:**
            - This function prevents the closing of specified channels and analog backplane relays.

        **TSP Syntax:**

        ::

            - channel.setforbidden()

        Args:
            channel_list: A string that lists the channels and analog backplane relays to make
                forbidden to close.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setforbidden("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setforbidden()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setlabel(self, channel_list: str, labelname: str) -> None:
        """Run the ``channel.setlabel()`` function.

        **Description:**
            - This function sets the label associated with a channel.

        **TSP Syntax:**

        ::

            - channel.setlabel()

        Args:
            channel_list: A string that lists the channel to which to set the label; the string
                cannot contain spaces.
            labelname: A string that contains the label for the channel in channelList, up to 19
                characters.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setlabel("{channel_list}", "{labelname}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setlabel()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setmatch(
        self,
        channel_list: str,
        match_value: str,
        mask: Optional[str] = None,
        width: Optional[str] = None,
    ) -> None:
        """Run the ``channel.setmatch()`` function.

        **Description:**
            - This function sets the match value on a digital input or totalizer channel.

        **TSP Syntax:**

        ::

            - channel.setmatch()

        Args:
            channel_list: String that specifies the channels to query, using normal channel list
                syntax.
            match_value: Channel value to compare on the specified channel.
            mask (optional): Value to specify the bits used to mask matchValue.
            width (optional): Value that specifies matches over multiple consecutive channels
                (default 1).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{channel_list}"',
                    match_value,
                    mask,
                    width,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.setmatch({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setmatch()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setmatchtype(self, channel_list: str, type_: str) -> None:
        """Run the ``channel.setmatchtype()`` function.

        **Description:**
            - This function sets the match type on a channel.

        **TSP Syntax:**

        ::

            - channel.setmatchtype()

        Args:
            channel_list: String specifying the channels to set, using normal channel list syntax.
            type_: A value for setting the match operation used on this specific channel.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setmatchtype("{channel_list}", {type_})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setmatchtype()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setmode(self, channel_list: str, mode: str) -> None:
        """Run the ``channel.setmode()`` function.

        **Description:**
            - This function sets the mode of operation of a channel.

        **TSP Syntax:**

        ::

            - channel.setmode()

        Args:
            channel_list: String specifying the channels to set, using normal channel list syntax.
            mode: The value that sets the mode of operation for a channel; see Details.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setmode("{channel_list}", {mode})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setmode()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setoutputenable(self, channel_list: str, state: str) -> None:
        """Run the ``channel.setoutputenable()`` function.

        **Description:**
            - This function sets the output enable attribute on a channel.

        **TSP Syntax:**

        ::

            - channel.setoutputenable()

        Args:
            channel_list: String specifying the channels to set, using normal channel list syntax.
            state: A value representing the state of the output of the channel.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setoutputenable("{channel_list}", {state})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setoutputenable()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setpole(self, channel_list: str, value: str) -> None:
        """Run the ``channel.setpole()`` function.

        **Description:**
            - This function specifies the pole setting for a list of channels.

        **TSP Syntax:**

        ::

            - channel.setpole()

        Args:
            channel_list: String specifying the channels to set, using normal channel list syntax.
            value: Pole setting for the channels in channelList. Use the following.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setpole("{channel_list}", {value})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setpole()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setstatelatch(self, channel_list: str, state_latch_mask: str) -> None:
        """Run the ``channel.setstatelatch()`` function.

        **Description:**
            - This function sets the state indicators to either latching or nonlatching.

        **TSP Syntax:**

        ::

            - channel.setstatelatch()

        Args:
            channel_list: The channels to set, using standard channel naming.
            state_latch_mask: A value specifying the indicators to latch.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setstatelatch("{channel_list}", {state_latch_mask})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setstatelatch()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def write(self, channel_list: str, value: str, width: Optional[str] = None) -> None:
        """Run the ``channel.write()`` function.

        **Description:**
            - This function writes a value to a channel.

        **TSP Syntax:**

        ::

            - channel.write()

        Args:
            channel_list: The channels to set, using standard channel naming.
            value: The value to be written to the channel (must be decimal value).
            width (optional): Value that specifies the channel width of the write.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{channel_list}"',
                    value,
                    width,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.write({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.write()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The filesystem commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - FILESystem:COPy {<source file path>,<destination file path>}
    - FILESystem:CWD {<new working directory path>}
    - FILESystem:DELEte <file path>
    - FILESystem:DIR?
    - FILESystem:MKDir <directory path>
    - FILESystem:PRInt <filepath>, GPIb
    - FILESystem:READFile <filepath>
    - FILESystem:REName <old file path>,<new file path>
    - FILESystem:RMDir <directory path>
    - FILESystem:WRITEFile <file path>,<data>
    - FILESystem?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class FilesystemWritefile(SCPICmdWrite):
    """The ``FILESystem:WRITEFile`` command.

    **Description:**
        - This command (no query form) copies the block data from the GPIB port to a named file.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:WRITEFile value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:WRITEFile <file path>,<data>

    **Info:**
        - ``<file path>`` is a quoted string that defines the file name and path. If the file path
          is within the current working directory, you need only specify the file name.
        - ``<data>`` can be either DEFINITE LENGTH or INDEFINITE LENGTH ARBITRARY BLOCK PROGRAM DATA
          encoding as described in IEEE 488.2.
    """


class FilesystemRmdir(SCPICmdWrite):
    """The ``FILESystem:RMDir`` command.

    **Description:**
        - Deletes a named directory. This command deletes the specified directory and all of its
          contents. The directory must not be a read-only directory.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:RMDir value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:RMDir <directory path>

    **Info:**
        - ``<directory path>`` is a quoted string that defines the directory name and path. If the
          file path is within the current working directory, you need only specify the file name.
    """


class FilesystemRename(SCPICmdWrite):
    """The ``FILESystem:REName`` command.

    **Description:**
        - Assigns a new name to an existing file.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:REName value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:REName <old file path>,<new file path>

    **Info:**
        - ``<old file path>`` is a quoted string that defines the file name and path. If the file
          path is within the current working directory, you need only specify the file name.
        - ``<new file path>`` is a quoted string that defines the file name and path. If the file
          path is within the current working directory, you need only specify the file name.
    """


class FilesystemReadfile(SCPICmdWrite):
    """The ``FILESystem:READFile`` command.

    **Description:**
        - This command (no query form) prints a named file to a named port. It is identical to the
          ``FILESYSTEM:PRINT`` command. This command is not IEEE 488.2 compliant for the following
          reasons: The command produces output on the 488.2 bus. Queries, not commands, are the only
          message units that should produce output on the 488.2 bus. Errors might not be recognized
          if they occur while data is being output in response to this command. This will be evident
          if a command is sent to the instrument before all of the data generated by this command is
          read. In such cases, you should send a device clear message to the instrument. The command
          does not use a standard 488.2 output format. The closest defined format is the indefinite
          format of ARBITRARY BLOCK PROGRAM DATA encoding. However, this command output omits the
          initial '#0' characters needed for that format. Newlines, ASCII value 10, can be included
          in the output data. Output on the bus can be terminated by newlines. The output of this
          command is only terminated by <EOI>.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:READFile value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:READFile <filepath>

    **Info:**
        - ``<file path>`` is a quoted string that defines the file name and path. If the file path
          is within the current working directory, you need only specify the file name.
    """


class FilesystemPrint(SCPICmdWrite):
    """The ``FILESystem:PRInt`` command.

    **Description:**
        - This command (no query form) prints a named file to a named port. This command is the same
          as the ``FILESYSTEM:READFILE`` command. This command is not IEEE 488.2 compliant for the
          following reasons: The command produces output on the 488.2 bus. Queries, not commands,
          are the only message units that should produce output on the 488.2 bus. Errors might not
          be recognized if they occur while data is being output in response to this command. This
          will be evident if a command is sent to the instrument before all of the data generated by
          this command is read. In such cases, you should send a device clear message to the
          instrument. The command does not use a standard 488.2 output format. The closest defined
          format is the indefinite format of ARBITRARY BLOCK PROGRAM DATA encoding. However, this
          command output omits the initial '#0' characters needed for that format. Newlines, ASCII
          value 10, can be included in the output data. Output on the bus can be terminated by
          newlines. The output of this command is only terminated by <EOI>.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:PRInt value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:PRInt <filepath>, GPIb

    **Info:**
        - ``<file path>`` is a quoted string that defines the file name and path. If the file path
          is within the current working directory, you need only specify the file name.
        - ``GPIb`` specifies that the hard copy is sent out of the GPIB port. GPIb is the only port
          allowed for this command.
    """


class FilesystemMkdir(SCPICmdWrite):
    """The ``FILESystem:MKDir`` command.

    **Description:**
        - Creates a new folder.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:MKDir value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:MKDir <directory path>

    **Info:**
        - ``<directory path>`` is a quoted string that specifies the directory to create.
    """


class FilesystemDir(SCPICmdRead):
    """The ``FILESystem:DIR`` command.

    **Description:**
        - This query-only command returns a comma separated list of quoted strings. Each string
          contains the name of a file or directory in the folder referred to by the
          ``FILESYSTEM:CWD`` command.

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem:DIR?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:DIR?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - FILESystem:DIR?
    """


class FilesystemDelete(SCPICmdWrite):
    r"""The ``FILESystem:DELEte`` command.

    **Description:**
        - This command deletes a named file. If you specify a directory name, it will delete the
          directory and all of its contents, the same as the RMDir command. You can also specify the
          filename as \*.* to delete all of the files in the current or specified directory.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:DELEte value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:DELEte <file path>

    **Info:**
        - ``<file path>`` is a quoted string that defines the file name and path. If the file path
          is within the current working directory, you need only specify the file name.
        - ``*.*`` will delete all files and subdirectories within the current working directory.
    """


class FilesystemCwd(SCPICmdWrite):
    """The ``FILESystem:CWD`` command.

    **Description:**
        - This command specifies the current working directory (CWD) for FILESystem commands. The
          default working directory is 'E:/'. Anytime you use this command to change the directory,
          the directory that you specify is retained as the current working directory until you
          either change the directory or you delete the directory. If you delete the current working
          directory, the oscilloscope resets current working directory to the default directory the
          next time the oscilloscope is powered on or the next time you execute a file system
          command. This command supports the permutations of file and directory names supported by
          Microsoft Windows: Relative path names; for example, './Temp' Absolute path names; for
          example, 'E:/MyWaveform' Implied relative path names; for example 'newfile.txt' becomes
          'E:/TekScope/newfile.txt' if the current working directory is 'E:/TekScope'

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:CWD value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:CWD {<new working directory path>}

    **Info:**
        - ``<new working directory path>`` is a quoted string that defines the current working; a
          directory name can be up to 128 characters.
    """


class FilesystemCopy(SCPICmdWrite):
    """The ``FILESystem:COPy`` command.

    **Description:**
        - This command (no query form) copies a named file to a new file. The new file might be in a
          totally separate directory than the old file. You can only copy one file at a time using
          this command. Wild card characters are not allowed.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:COPy value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:COPy {<source file path>,<destination file path>}

    **Info:**
        - ``<source file path>`` is a quoted string that defines the file name and path. If the file
          path is within the current working directory, you need only specify the file name.
        - ``<destination file path>`` is a quoted string that defines the file name and path. If the
          file path is within the current working directory, you need only specify the file name.
    """


#  pylint: disable=too-many-instance-attributes
class Filesystem(SCPICmdRead):
    """The ``FILESystem`` command.

    **Description:**
        - This query-only command returns the directory listing of the current working directory.
          This query is the same as the ``FILESystem:DIR?`` query.

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - FILESystem?

    Properties:
        - ``.copy``: The ``FILESystem:COPy`` command.
        - ``.cwd``: The ``FILESystem:CWD`` command.
        - ``.delete``: The ``FILESystem:DELEte`` command.
        - ``.dir``: The ``FILESystem:DIR`` command.
        - ``.mkdir``: The ``FILESystem:MKDir`` command.
        - ``.print``: The ``FILESystem:PRInt`` command.
        - ``.readfile``: The ``FILESystem:READFile`` command.
        - ``.rename``: The ``FILESystem:REName`` command.
        - ``.rmdir``: The ``FILESystem:RMDir`` command.
        - ``.writefile``: The ``FILESystem:WRITEFile`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "FILESystem") -> None:
        super().__init__(device, cmd_syntax)
        self._copy = FilesystemCopy(device, f"{self._cmd_syntax}:COPy")
        self._cwd = FilesystemCwd(device, f"{self._cmd_syntax}:CWD")
        self._delete = FilesystemDelete(device, f"{self._cmd_syntax}:DELEte")
        self._dir = FilesystemDir(device, f"{self._cmd_syntax}:DIR")
        self._mkdir = FilesystemMkdir(device, f"{self._cmd_syntax}:MKDir")
        self._print = FilesystemPrint(device, f"{self._cmd_syntax}:PRInt")
        self._readfile = FilesystemReadfile(device, f"{self._cmd_syntax}:READFile")
        self._rename = FilesystemRename(device, f"{self._cmd_syntax}:REName")
        self._rmdir = FilesystemRmdir(device, f"{self._cmd_syntax}:RMDir")
        self._writefile = FilesystemWritefile(device, f"{self._cmd_syntax}:WRITEFile")

    @property
    def copy(self) -> FilesystemCopy:
        """Return the ``FILESystem:COPy`` command.

        **Description:**
            - This command (no query form) copies a named file to a new file. The new file might be
              in a totally separate directory than the old file. You can only copy one file at a
              time using this command. Wild card characters are not allowed.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:COPy value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:COPy {<source file path>,<destination file path>}

        **Info:**
            - ``<source file path>`` is a quoted string that defines the file name and path. If the
              file path is within the current working directory, you need only specify the file
              name.
            - ``<destination file path>`` is a quoted string that defines the file name and path. If
              the file path is within the current working directory, you need only specify the file
              name.
        """
        return self._copy

    @property
    def cwd(self) -> FilesystemCwd:
        """Return the ``FILESystem:CWD`` command.

        **Description:**
            - This command specifies the current working directory (CWD) for FILESystem commands.
              The default working directory is 'E:/'. Anytime you use this command to change the
              directory, the directory that you specify is retained as the current working directory
              until you either change the directory or you delete the directory. If you delete the
              current working directory, the oscilloscope resets current working directory to the
              default directory the next time the oscilloscope is powered on or the next time you
              execute a file system command. This command supports the permutations of file and
              directory names supported by Microsoft Windows: Relative path names; for example,
              './Temp' Absolute path names; for example, 'E:/MyWaveform' Implied relative path
              names; for example 'newfile.txt' becomes 'E:/TekScope/newfile.txt' if the current
              working directory is 'E:/TekScope'

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:CWD value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:CWD {<new working directory path>}

        **Info:**
            - ``<new working directory path>`` is a quoted string that defines the current working;
              a directory name can be up to 128 characters.
        """
        return self._cwd

    @property
    def delete(self) -> FilesystemDelete:
        r"""Return the ``FILESystem:DELEte`` command.

        **Description:**
            - This command deletes a named file. If you specify a directory name, it will delete the
              directory and all of its contents, the same as the RMDir command. You can also specify
              the filename as \*.* to delete all of the files in the current or specified directory.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:DELEte value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:DELEte <file path>

        **Info:**
            - ``<file path>`` is a quoted string that defines the file name and path. If the file
              path is within the current working directory, you need only specify the file name.
            - ``*.*`` will delete all files and subdirectories within the current working directory.
        """
        return self._delete

    @property
    def dir(self) -> FilesystemDir:
        """Return the ``FILESystem:DIR`` command.

        **Description:**
            - This query-only command returns a comma separated list of quoted strings. Each string
              contains the name of a file or directory in the folder referred to by the
              ``FILESYSTEM:CWD`` command.

        **Usage:**
            - Using the ``.query()`` method will send the ``FILESystem:DIR?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:DIR?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - FILESystem:DIR?
        """
        return self._dir

    @property
    def mkdir(self) -> FilesystemMkdir:
        """Return the ``FILESystem:MKDir`` command.

        **Description:**
            - Creates a new folder.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:MKDir value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:MKDir <directory path>

        **Info:**
            - ``<directory path>`` is a quoted string that specifies the directory to create.
        """
        return self._mkdir

    @property
    def print(self) -> FilesystemPrint:
        """Return the ``FILESystem:PRInt`` command.

        **Description:**
            - This command (no query form) prints a named file to a named port. This command is the
              same as the ``FILESYSTEM:READFILE`` command. This command is not IEEE 488.2 compliant
              for the following reasons: The command produces output on the 488.2 bus. Queries, not
              commands, are the only message units that should produce output on the 488.2 bus.
              Errors might not be recognized if they occur while data is being output in response to
              this command. This will be evident if a command is sent to the instrument before all
              of the data generated by this command is read. In such cases, you should send a device
              clear message to the instrument. The command does not use a standard 488.2 output
              format. The closest defined format is the indefinite format of ARBITRARY BLOCK PROGRAM
              DATA encoding. However, this command output omits the initial '#0' characters needed
              for that format. Newlines, ASCII value 10, can be included in the output data. Output
              on the bus can be terminated by newlines. The output of this command is only
              terminated by <EOI>.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:PRInt value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:PRInt <filepath>, GPIb

        **Info:**
            - ``<file path>`` is a quoted string that defines the file name and path. If the file
              path is within the current working directory, you need only specify the file name.
            - ``GPIb`` specifies that the hard copy is sent out of the GPIB port. GPIb is the only
              port allowed for this command.
        """
        return self._print

    @property
    def readfile(self) -> FilesystemReadfile:
        """Return the ``FILESystem:READFile`` command.

        **Description:**
            - This command (no query form) prints a named file to a named port. It is identical to
              the ``FILESYSTEM:PRINT`` command. This command is not IEEE 488.2 compliant for the
              following reasons: The command produces output on the 488.2 bus. Queries, not
              commands, are the only message units that should produce output on the 488.2 bus.
              Errors might not be recognized if they occur while data is being output in response to
              this command. This will be evident if a command is sent to the instrument before all
              of the data generated by this command is read. In such cases, you should send a device
              clear message to the instrument. The command does not use a standard 488.2 output
              format. The closest defined format is the indefinite format of ARBITRARY BLOCK PROGRAM
              DATA encoding. However, this command output omits the initial '#0' characters needed
              for that format. Newlines, ASCII value 10, can be included in the output data. Output
              on the bus can be terminated by newlines. The output of this command is only
              terminated by <EOI>.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:READFile value``
              command.

        **SCPI Syntax:**

        ::

            - FILESystem:READFile <filepath>

        **Info:**
            - ``<file path>`` is a quoted string that defines the file name and path. If the file
              path is within the current working directory, you need only specify the file name.
        """
        return self._readfile

    @property
    def rename(self) -> FilesystemRename:
        """Return the ``FILESystem:REName`` command.

        **Description:**
            - Assigns a new name to an existing file.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:REName value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:REName <old file path>,<new file path>

        **Info:**
            - ``<old file path>`` is a quoted string that defines the file name and path. If the
              file path is within the current working directory, you need only specify the file
              name.
            - ``<new file path>`` is a quoted string that defines the file name and path. If the
              file path is within the current working directory, you need only specify the file
              name.
        """
        return self._rename

    @property
    def rmdir(self) -> FilesystemRmdir:
        """Return the ``FILESystem:RMDir`` command.

        **Description:**
            - Deletes a named directory. This command deletes the specified directory and all of its
              contents. The directory must not be a read-only directory.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:RMDir value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:RMDir <directory path>

        **Info:**
            - ``<directory path>`` is a quoted string that defines the directory name and path. If
              the file path is within the current working directory, you need only specify the file
              name.
        """
        return self._rmdir

    @property
    def writefile(self) -> FilesystemWritefile:
        """Return the ``FILESystem:WRITEFile`` command.

        **Description:**
            - This command (no query form) copies the block data from the GPIB port to a named file.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:WRITEFile value``
              command.

        **SCPI Syntax:**

        ::

            - FILESystem:WRITEFile <file path>,<data>

        **Info:**
            - ``<file path>`` is a quoted string that defines the file name and path. If the file
              path is within the current working directory, you need only specify the file name.
            - ``<data>`` can be either DEFINITE LENGTH or INDEFINITE LENGTH ARBITRARY BLOCK PROGRAM
              DATA encoding as described in IEEE 488.2.
        """
        return self._writefile



"""Module containing an inner class to use for an API for IEEE 488.2 commands."""
from typing import Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class IEEE4882Commands:
    """An inner class containing an API for all the IEEE 488.2 SCPI commands."""

    ################################################################################################
    # Magic Methods
    ################################################################################################
    def __init__(self, pi_device: "PIDevice") -> None:
        """Create an instance of the IEEE 488.2 commands class."""
        self._pi_device = pi_device

    ################################################################################################
    # Public Methods
    ################################################################################################
    def cls(self) -> None:
        r"""Send the ``*CLS`` (Clear Status) command.

        This command (no query form) clears the following:

        - Event Queue
        - Standard Event Status Register
        - Status Byte Register (except the MAV bit)

        If the ``*CLS`` command immediately follows an <EOI> end-of-message terminator,
        the Output Queue and MAV bit (Status Byte Register bit 4) are also cleared.
        MAV indicates that information is in the output queue.
        The device clear (DCL) control message will clear the output queue and thus MAV.
        ``*CLS`` does not clear the output queue or MAV.

        ``*CLS`` can suppress a Service Request that is to be generated by an ``*OPC``.
        """
        self._pi_device.write("*CLS")

    def ese(self, value: Optional[int] = None) -> str:
        """Send or query the ``*ESE`` (Event Status Enable) command.

        This command sets and queries the bits in the Event Status Enable Register (ESER).
        The ESER prevents events from being reported to the Status Byte Register (STB).

        The power-on default for the ESER is 0 if ``*PSC`` is 1. If ``*PSC`` is 0, the ESER
        maintains the previous power cycle value through the current power cycle.

        Args:
            value: An integer specifying the binary bits of the ESER, from 0 to 255.

        Returns:
            The output of the query.

        Raises:
            ValueError: Indicates that the input value is not within the accepted range.
        """
        if value is not None:
            if not 0 <= value <= 255:  # noqa: PLR2004
                msg = f"{value=} is not a valid value. The value must be between 0 and 255."
                raise ValueError(msg)
            return self._pi_device.set_and_check("*ESE", value)
        return self._pi_device.query("*ESE?")

    def esr(self) -> str:
        """Query the ``*ESR?`` (Standard Event Status Register) command.

        This query-only command returns the contents of the Standard Event Status Register (SESR).
        ``*ESR?`` also clears the SESR (since reading the SESR clears it).

        Returns:
            The output of the query.
        """
        return self._pi_device.query("*ESR?")

    def idn(self) -> str:
        """Query the ``*IDN?`` (Identification) command.

        This query-only command returns the instrument identification code.

        Returns:
            The output of the query.
        """
        return self._pi_device.query("*IDN?")

    def lrn(self) -> str:
        """Query the ``*LRN?`` (Learn) command.

        This query-only command returns the commands that list the instrument settings.

        Returns:
            The output of the query.
        """
        return self._pi_device.query("*LRN?")

    def opc(self, write: bool = False) -> str:
        r"""Send or query the ``*OPC`` (Operation Complete) command.

        The ``*OPC`` command generates an operation complete event which propagates
        through the Standard Event Status Register (SESR) when all pending commands
        that block OPC are complete.

        The ``*OPC?`` query returns "1" after all blocking OPC operations are complete.
        The ``*OPC?`` query response is not available to read until all pending operations finish,
        so an appropriate timeout value should be used.

        Args:
            write: A boolean indicating if the ``*OPC`` should be set rather than queried.

        Returns:
            The output of the query, otherwise an empty string.
        """
        if write:
            self._pi_device.write("*OPC")
            return ""
        return self._pi_device.query("*OPC?")

    def opt(self) -> str:
        """Query the ``*OPT?`` (Options) command.

        This query-only command returns a comma separated list of installed options as an
        arbitrary ASCII string (no quotes) of the form:

        ``<optionCode>:<optionDescription>,<optionCode>:<optionDescription>...``

        Returns:
            The output of the query.
        """
        return self._pi_device.query("*OPT?")

    def psc(self, value: Optional[bool] = None) -> str:
        """Send or query the ``*PSC`` (Power-on Status Clear) command.

        This command sets and queries the power-on status flag that controls the automatic
        power-on handling of the SRER, and ESER registers. When ``*PSC`` is true,
        the SRER and ESER registers are set to 0 at power-on. When ``*PSC`` is false,
        the current values in the SRER and ESER registers are preserved in nonvolatile memory
        when power is shut off and are restored at power-on.

        Args:
            value: A boolean indicating the status to set the power-on status flag to.

        Returns:
            The output of the query.
        """
        if value is not None:
            return self._pi_device.set_and_check("*PSC", int(value))

        return self._pi_device.query("*PSC?")

    def rst(self) -> None:
        r"""Send the ``*RST`` (Reset) command.

        This command (no query form) does the following:

        - Recalls the default instrument setup.

        The ``*RST`` command does not change the following:

        - The Power-on Status Clear Flag (``*PSC`` command).
        - The Event Status Enable Register (``*ESE`` command).
        - The Service Request Enable Register (``*SRE`` command).

        ``*RST`` only resets the programmable interface settings, it does not change any
        user interface settings.
        """
        self._pi_device.write("*RST")

    def sre(self, value: Optional[int] = None) -> str:
        """Send or query the ``*SRE`` (Service Request Enable) command.

        The ``*SRE`` (Service Request Enable) command sets and queries the bits in the
        Service Request Enable Register.

        Args:
            value: An integer specifying the binary bits of the SRER, from 0 to 255.

        Returns:
            The output of the query.

        Raises:
            ValueError: Indicates that the input value is not within the accepted range.
        """
        if value is not None:
            if not 0 <= value <= 255:  # noqa: PLR2004
                msg = f"{value=} is not a valid value. The value must be between 0 and 255."
                raise ValueError(msg)
            return self._pi_device.set_and_check("*SRE", value)
        return self._pi_device.query("*SRE?")

    def stb(self) -> str:
        """Query the ``*STB?`` (Status Byte) command.

        The ``*STB?`` (Read Status Byte) query returns the contents of the
        Status Byte Register (SBR) using the Master Summary Status (MSS) bit.

        Returns:
            The output of the query.
        """
        return self._pi_device.query("*STB?")

    def tst(self) -> str:
        """Send the ``*TST?`` (Self-Test) command.

        Tests (self-test) the interface and returns the result.

        Returns:
            The output of the query.
        """
        return self._pi_device.query("*TST?")

    def wai(self) -> None:
        """Send the ``*WAI`` (Wait) command.

        The ``*WAI`` (Wait) command (no query form) prevents the instrument from executing further
        commands or queries until all pending commands that generate an OPC message are complete.
        """
        self._pi_device.write("*WAI")


class TSPIEEE4882Commands(IEEE4882Commands):
    """An inner class containing an API for all the IEEE488.2 TSP commands."""

    ################################################################################################
    # Public Methods
    ################################################################################################
    def cls(self) -> None:
        r"""Send Clear Status commands (``errorqueue.clear()``, ``status.clear()``).

        This command (no query form) clears the following:

        - Event Queue
        - Standard Event Status Register
        - Status Byte Register
        """
        self._pi_device.write("errorqueue.clear()")
        self._pi_device.write("status.reset()")

    def ese(self, value: Optional[int] = None) -> str:
        """Send or query the Event Status Enable (``status.standard.enable``) command.

        This command sets and queries the bits in the Event Status Enable Register (ESER).
        The ESER prevents events from being reported to the Status Byte Register (STB).

        Args:
            value: An integer specifying the binary bits of the ESER, from 0 to 255.

        Returns:
            The output of the query.

        Raises:
            ValueError: Indicates that the input value is not within the accepted range.
        """
        if value is not None:
            if not 0 <= value <= 255:  # noqa: PLR2004
                msg = f"{value=} is not a valid value. The value must be between 0 and 255."
                raise ValueError(msg)
            return self._pi_device.set_and_check("status.standard.enable", value)
        return self._pi_device.query("print(status.standard.enable)")

    def esr(self) -> str:
        """Query the Standard Event Status Register (``print(status.standard.event)``) command.

        This query-only command returns the contents of the Standard Event Status Register (SESR).
        ``print(status.standard.event)`` also clears the SESR (since reading the SESR clears it).

        Returns:
            The output of the query.
        """
        return self._pi_device.query("print(status.standard.event)")

    def opc(self, write: bool = False) -> str:
        r"""Send or query the Operation Complete command.

        The OPC command generates an operation complete event which propagates
        through the Standard Event Status Register (SESR) when all pending commands
        that block OPC are complete.

        Args:
            write: A boolean indicating if the command should be set rather than queried.

        Returns:
            The output of the query, otherwise an empty string.
        """
        if write:
            self._pi_device.write("opc()")
            return ""
        self.wai()
        return self._pi_device.query("print([[1]])")

    def rst(self) -> None:
        r"""Send the Reset (``reset()``) command."""
        self._pi_device.write("reset()")

    def sre(self, value: Optional[int] = None) -> str:
        """Send or query the Service Request Enable (``status.request_enable``) command.

        The ``status.request_enable`` (Service Request Enable) command sets and
        queries the bits in the Service Request Enable Register.

        Args:
            value: An integer specifying the binary bits of the SRER, from 0 to 255.

        Returns:
            The output of the query.

        Raises:
            ValueError: Indicates that the input value is not within the accepted range.
        """
        if value is not None:
            if not 0 <= value <= 255:  # noqa: PLR2004
                msg = f"{value=} is not a valid value. The value must be between 0 and 255."
                raise ValueError(msg)
            return self._pi_device.set_and_check("status.request_enable", value)
        return self._pi_device.query("print(status.request_enable)")

    def stb(self) -> str:
        """Query the Status Byte (``print(status.condition)``) command.

        Returns:
            The output of the query.
        """
        return self._pi_device.query("print(status.condition)")

    def tst(self) -> str:
        """Send the (``print([[0]])``) (Self-Test) command.

        Tests (self-test) the interface and returns the result.

        Returns:
            The output of the query.
        """
        return self._pi_device.query("print([[0]])")

    def wai(self) -> None:
        """Send the ``waitcomplete()`` command."""
        self._pi_device.write("waitcomplete()")


class LegacyTSPIEEE4882Commands(TSPIEEE4882Commands):
    """An inner class containing an API for all the IEEE 488.2 commands for legacy TSP models."""

    ################################################################################################
    # Public Methods
    ################################################################################################
    def cls(self) -> None:
        r"""Send Clear Status commands (``eventlog.clear()``, ``status.clear()``).

        This command (no query form) clears the following:

        - Event Queue
        - Standard Event Status Register
        - Status Byte Register
        """
        self._pi_device.write("eventlog.clear()")
        self._pi_device.write("status.clear()")



"""The diggrp commands module.

These commands are used in the following models:
LPD6, MSO4, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - DIGGRP<x>:D<x>:THReshold <NR3>
    - DIGGRP<x>:D<x>:THReshold?
"""
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDigitalBit,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class DiggrpItemDigitalBitThreshold(SCPICmdWrite, SCPICmdRead):
    """The ``DIGGRP<x>:D<x>:THReshold`` command.

    **Description:**
        - Sets or queries the threshold level in volts for the specified digital channel. If the
          source channel doesn't exist, a hardware missing error event is set.

    **Usage:**
        - Using the ``.query()`` method will send the ``DIGGRP<x>:D<x>:THReshold?`` query.
        - Using the ``.verify(value)`` method will send the ``DIGGRP<x>:D<x>:THReshold?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIGGRP<x>:D<x>:THReshold value``
          command.

    **SCPI Syntax:**

    ::

        - DIGGRP<x>:D<x>:THReshold <NR3>
        - DIGGRP<x>:D<x>:THReshold?

    **Info:**
        - ``DIGGRP<x>`` is the channel number.
        - ``D<x>`` is the digital channel bit number (0-7).
        - ``<NR3>`` is the threshold level in volts for the specified digital channel.
    """


class DiggrpItemDigitalBit(ValidatedDigitalBit, SCPICmdRead):
    """The ``DIGGRP<x>:D<x>`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``DIGGRP<x>:D<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``DIGGRP<x>:D<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **Info:**
        - ``DIGGRP<x>`` is the channel number.
        - ``D<x>`` is the digital channel bit number (0-7).

    Properties:
        - ``.threshold``: The ``DIGGRP<x>:D<x>:THReshold`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._threshold = DiggrpItemDigitalBitThreshold(device, f"{self._cmd_syntax}:THReshold")

    @property
    def threshold(self) -> DiggrpItemDigitalBitThreshold:
        """Return the ``DIGGRP<x>:D<x>:THReshold`` command.

        **Description:**
            - Sets or queries the threshold level in volts for the specified digital channel. If the
              source channel doesn't exist, a hardware missing error event is set.

        **Usage:**
            - Using the ``.query()`` method will send the ``DIGGRP<x>:D<x>:THReshold?`` query.
            - Using the ``.verify(value)`` method will send the ``DIGGRP<x>:D<x>:THReshold?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIGGRP<x>:D<x>:THReshold value``
              command.

        **SCPI Syntax:**

        ::

            - DIGGRP<x>:D<x>:THReshold <NR3>
            - DIGGRP<x>:D<x>:THReshold?

        **Info:**
            - ``DIGGRP<x>`` is the channel number.
            - ``D<x>`` is the digital channel bit number (0-7).
            - ``<NR3>`` is the threshold level in volts for the specified digital channel.
        """
        return self._threshold


class DiggrpItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``DIGGRP<x>`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``DIGGRP<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``DIGGRP<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **Info:**
        - ``DIGGRP<x>`` is the channel number.

    Properties:
        - ``.d``: The ``DIGGRP<x>:D<x>`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "DIGGRP<x>") -> None:
        super().__init__(device, cmd_syntax)
        self._d: Dict[int, DiggrpItemDigitalBit] = DefaultDictPassKeyToFactory(
            lambda x: DiggrpItemDigitalBit(device, f"{self._cmd_syntax}:D{x}")
        )

    @property
    def d(self) -> Dict[int, DiggrpItemDigitalBit]:
        """Return the ``DIGGRP<x>:D<x>`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``DIGGRP<x>:D<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``DIGGRP<x>:D<x>?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **Info:**
            - ``DIGGRP<x>`` is the channel number.
            - ``D<x>`` is the digital channel bit number (0-7).

        Sub-properties:
            - ``.threshold``: The ``DIGGRP<x>:D<x>:THReshold`` command.
        """
        return self._d



"""The opcextended commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - OPCEXtended {<NR1>|OFF|ON}
    - OPCEXtended?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Opcextended(SCPICmdWrite, SCPICmdRead):
    """The ``OPCEXtended`` command.

    **Description:**
        - This command sets or queries the behavior of OPC commands and queries. When enabled,
          operations referenced in the ``*OPC`` command description notify when their overlapped
          functionality has completed. When disabled, the operations notify as they have in the past
          (only once updated in the instrument state database). Command synchronization Operation PI
          sequence Single sequence with ttOff ``:ACQUIRE:STOPAFTER SEQUENCE``
          ``:ACQUIRE:STATE 1``;``*OPC?``;``:WFMOUTPRE:XZERO?`` Single sequence with Measurement
          Annotation ``:ACQUIRE:STOPAFTER SEQUENCE``;``:MEASUREMENT:MEAS1:STATE 1``;TYPE PK2PK
          ``:ACQUIRE:STATE 1``;``*OPC?``;``:MEASUREMENT:ANNOTATION:X1?`` Single sequence with
          Cursors ``:ACQUIRE:STOPAFTER SEQUENCE``;``:CURSOR:FUNCTION WAVEFORM``;SOURCE CH1;STATE 1
          ``:ACQUIRE:STATE 1``;``*OPC?`` Single sequence with Math
          ``:ACQUIRE:STOPAFTER SEQUENCE``;``:MATH1:DEFINE`` 'Ch1``*Ch2``';``:SELECT:MATH1 1``
          ``:ACQUIRE:STATE 1``;``*OPC?`` Default setup followed by Save Waveform ``*RST``;``*OPC?``
          ``:SAVE:WAVEFORM`` CH1,REF1;``*WAI`` ``:SELECT:REF1 1`` Math On during Acq Run mode
          ``:HORIZONTAL:MODE MANUAL``;RECORDLENGTH 2500000 ``:MATH1:DEFINE``
          'CH1``*CH1``';``:SELECT:MATH1 1`` ``:DATA:ENCDG ASCII``;SOURCE REF1;START 1;STOP 10
          ``:SELECT:MATH1 0`` {Wait a couple sec..longer in release mode?}
          ``:SELECT:MATH1 1``;``*WAI``;``:CURVE?`` Save Math to Ref
          ``:HORIZONTAL:MODE MANUAL``;RECORDLENGTH 2500000 ``:MATH1:DEFINE``
          'CH1``*CH1``';``:SELECT:MATH1 1``;``*WAI``; ``:SAVE:WAVEFORM``
          MATH1,REF1;``:SELECT:REF1 1`` ``:DATA:ENCDG ASCII``;SOURCE REF1;START 1;STOP 10 CURVE?
          Trigger state ``:ACQUIRE:STOPAFTER SEQUENCE``
          ``:ACQUIRE:STATE 1``;``*OPC?``;``:TRIGGER:STATE?`` Single sequence with Measurement
          ``:ACQUIRE:STOPAFTER SEQUENCE``;``:MEASUREMENT:MEAS1:STATE 1``;TYPE AMPLITUDE
          ``:ACQUIRE:STATE 1``;``*OPC?``;``:MEASUREMENT:MEAS1:VALUE?`` Single sequence with
          Measurement on Math
          ``:ACQUIRE:STOPAFTER SEQUENCE``;``:HORIZONTAL:MODE MANUAL``;RECORDLENGTH 2500000
          ``:MATH1:DEFINE`` 'CH1``*CH1``';``:SELECT:MATH1 1`` ``:MEASUREMENT:MEAS1:STATE 1``;TYPE
          AMPLITUDE;SOURCE MATH1 ``:ACQUIRE:STATE 1``;``*OPC?``;``:MEASUREMENT:MEAS1:VALUE?`` Acq
          Count ``*RST``;``*WAI``;``:ACQUIRE:NUMACQ?`` Acq state after single sequence
          ``:ACQUIRE:STOPAFTER SEQUENCE``;STATE 1;``*WAI``;``:ACQUIRE:STATE?``

    **Usage:**
        - Using the ``.query()`` method will send the ``OPCEXtended?`` query.
        - Using the ``.verify(value)`` method will send the ``OPCEXtended?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``OPCEXtended value`` command.

    **SCPI Syntax:**

    ::

        - OPCEXtended {<NR1>|OFF|ON}
        - OPCEXtended?

    **Info:**
        - ``ON`` turns on extended OPC behavior.
        - ``OFF`` turns off extended OPC behavior.
        - ``<NR1>`` = 0 turns off extended OPC behavior; any other value turns on extended OPC
          behavior.
    """

    def __init__(
        self, device: Optional["PIDevice"] = None, cmd_syntax: str = "OPCEXtended"
    ) -> None:
        super().__init__(device, cmd_syntax)
